/*
 * dumps an id database in a simple format.  Written partly so I could
 * make sure I understood the id format before making changes.
 */
/* Mark Moraes, DCS, University of Toronto */
#ifndef lint
static char rcsid[] = "$Header: /nfs/smoke/homes/smoke/quote/moraes/expt/id/RCS/dumpid.c,v 1.2 91/01/31 22:22:53 moraes Exp $";
#endif /*lint*/

#include <stdio.h>

#include "bool.h"
#include "id.h"
#include "bitops.h"
#include "extern.h"
#include "string.h"

#define MAXSTR 512

char *MyName;
#define progname MyName
int debug = 0;

/* For getopt() */
extern int optind;
extern char *optarg;

extern void error();

extern char *emalloc();
extern char *malloc();
extern char *realloc();
extern char *calloc();

FILE		*IdFILE;
char		*IdDir;
int		BitArraySize;
struct idhead	Idh;
struct idarg	*IdArgs;

int
main(argc, argv)
int argc;
char **argv;
{
	int c;
	int errs = 0;
	char *idFile = IDFILE;

	progname = argv[0] ? argv[0] : "(no-argv[0])";

	while((c = getopt(argc, argv, "f:d")) != EOF) {
		/* optarg has the argument if the option was followed by ':' */
		switch (c) {
		case 'd':
			debug++;
			break;
		case 'f':
			idFile = optarg;
			break;
		case '?':
			errs++;
			break;
		}
	}
	if (optind < argc || errs > 0) {
		(void) fprintf(stderr, "Usage: %s [-d] [-f file]\n", progname);
		exit(1);
	}
	process(idFile);
	return 0;
}

process(fname)
char *fname;
{
	struct idhead *idhp = &Idh;
	char *buf;
	int i;
	
	if ((IdDir = getDirToName(fname))== NULL) {
		filerr("getdir", fname);
		exit(1);
	}
	fname = spanPath(IdDir, fname);
	if ((IdFILE = idOpen(fname, "r", idhp)) == NULL)
		exit(1);
#define plural(x)  ((x) == 1 ? "" : "s")
	printf("%s is id version %d\n%d arg%s, %d path%s, %d identifier%s\n",
	       fname, idhp->idh_vers, idhp->idh_argc, plural(idhp->idh_argc),
	       idhp->idh_pthc, plural(idhp->idh_pthc),
	       idhp->idh_namc, plural(idhp->idh_namc));
	printf("%d byte%s per path vector entry, %d byte%s in max. entry\n",
	       idhp->idh_vecc, plural(idhp->idh_vecc),
	       idhp->idh_bsiz, plural(idhp->idh_bsiz));
	printf("args start at 0x%lx, names at 0x%lx, file ends at 0x%lx\n",
	       idhp->idh_argo, idhp->idh_namo, idhp->idh_endo);
	if (fseek(IdFILE, idhp->idh_argo, 0) < 0) {
		filerr("seek", fname);
		exit(1);
	}
	BitArraySize = (idhp->idh_pthc + 7) >> 3;
	printf("BitArraySize %d\n", BitArraySize);
	buf = malloc(idhp->idh_bsiz);	/* !! use emalloc */
	printf("Files: (names starting with - or + are options)\n");
	for(i = 0; i < idhp->idh_argc; i++) {
		fgets0(buf, idhp->idh_bsiz, IdFILE);
		if (((i & 0x00ff) == 0x00ff || (i & 0xff00) == 0xff00) &&
		    buf[0] == '\0')
			continue;
		puts(buf);
	}
	if (getc(IdFILE) != 0x00ff) {
		fprintf(stderr, "%s: path segment didn't end with 0xFF\n",
			progname);
		exit(1);
	}
	if (ftell(IdFILE) != idhp->idh_namo) {
		fprintf(stderr, "%s: not at start of names after paths\n",
			progname);
		exit(1);
	}
	printf("Words:\n");
	for(i = 0; i < idhp->idh_namc; i++) {
		int size;
		
		if ((size = fgets0(buf, idhp->idh_bsiz, IdFILE)) <= 0) {
			fprintf(stderr, "%s: premature end of file\n",
				progname);
			exit(1);
		}
		size++;
		size = getsFF(&buf[size], IdFILE);
		printf("%s\t%d\n", &buf[1], size / idhp->idh_vecc);
	}
	if (getc(IdFILE) != 0x00ff) {
		fprintf(stderr, "%s: name segment didn't end with 0xFF\n",
			progname);
		exit(1);
	}
	if (ftell(IdFILE) != idhp->idh_endo) {
		fprintf(stderr, "%s: not at end of file after names\n",
			progname);
		exit(1);
	}
}
